@@ -258,7 +258,7 @@ class BeagleMockCatalogue(object):
         hdulist.append(new_hdu)
 
         name = prepare_data_saving(file_name, overwrite=overwrite)
-        hdulist.writeto(name, clobber=overwrite)
+        hdulist.writeto(name, overwrite=overwrite)
 
         self.columns = new_hdu.columns
         self.data = new_hdu.data
@@ -425,7 +425,7 @@ class BeagleMockCatalogue(object):
         #plt.tight_layout()
 
         fig.savefig(name, dpi=None, facecolor='w', edgecolor='w',
-                orientation='portrait', papertype='a4', format="pdf",
+                orientation='portrait', format="pdf",
                 transparent=False, bbox_inches="tight", pad_inches=0.1)
 
         plt.close(fig)
@@ -717,7 +717,7 @@ class BeagleMockCatalogue(object):
         plt.tight_layout()
 
         fig.savefig(name, dpi=None, facecolor='w', edgecolor='w',
-                orientation='portrait', papertype='a4', format="pdf",
+                orientation='portrait', format="pdf",
                 transparent=False, bbox_inches="tight", pad_inches=0.1)
 
         plt.close(fig)
@@ -878,7 +878,7 @@ class BeagleMockCatalogue(object):
             #plt.tight_layout()
 
             fig.savefig(name, dpi=None, facecolor='w', edgecolor='w',
-                    orientation='portrait', papertype='a4', format="pdf",
+                    orientation='portrait', format="pdf",
                     transparent=False, bbox_inches="tight", pad_inches=0.1)
 
         plt.close(fig)
